import { enemyData } from "./enemyData.js";
import { characterData } from "./characterData.js";
import { propData } from "./propData.js";
import { levelStateStart, levelStateState as levelStateInfo, levelStateUpdate } from "./Outgoing/levelStatePartial.js";
import { Type } from "class-transformer";

export class levelState{
    id : string
    seed : number = -1
    hostId : number = -1
    completed : boolean = false
    isDungeon : boolean = false
    @Type(() => enemyData)
    enemies : enemyData[] = []
    @Type(() => propData)
    objects : propData[] = []
    @Type(() => characterData)
    players : characterData[] = []

    getRandomInt(max) {
        return Math.floor(Math.random() * max);
    }

    constructor(id, hostId){
        this.id = id
        this.hostId = hostId
        this.seed = this.getRandomInt(1000000)
        this.completed = false
    }

    copyStart(){
        return new levelStateStart(this)
    }

    copyUpdate(){
        return new levelStateUpdate(this)
    }

    copyState(){
        return new levelStateInfo(this)
    }

    isHostValid(){
        if(this.hostId == -1) return false
        if(this.players.length == 0) return false

        let playerIdx = this.players.findIndex(p=>p.id==this.hostId)
        if(playerIdx == -1) return false

        return true
    }

    addPlayer(player : characterData) : boolean{
        let playerIdx = this.players.findIndex(p=>p.id==player.id)
        if(playerIdx != -1) return false

        this.players.push(player)

        if(this.hostId == -1){
            this.hostId = this.players[0].id
        }

        return true
    }

    removePlayer(player : characterData) : boolean{
        this.players = this.players.filter(p=> p !== player)

        if(this.hostId == player.id){
            if(this.players.length > 0){
                this.hostId = this.players[0].id
            }
            else{
                this.hostId = -1;
            }
        }

        return true
    }
}